package com.ruoyi.sim.service.impl;

import java.util.List;

import cn.hutool.http.useragent.UserAgentUtil;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.sim.domain.Fault;
import com.ruoyi.sim.domain.TaskFault;
import com.ruoyi.sim.domain.vo.FaultTreeVo;
import com.ruoyi.sim.domain.vo.TaskVo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.sim.mapper.TaskMapper;
import com.ruoyi.sim.domain.Task;
import org.springframework.transaction.annotation.Transactional;

/**
 * 任务Service业务层处理
 *
 * @author tom
 * @date 2024-12-13
 */
@Service
public class TaskService {
    @Autowired
    private TaskMapper taskMapper;

    /**
     * 查询任务
     *
     * @param taskId 任务主键
     * @return 任务
     */
    public Task selectTaskByTaskId(Long taskId) {
        return taskMapper.selectTaskByTaskId(taskId);
    }

    /**
     * 查询任务列表
     *
     * @param task 任务
     * @return 任务
     */
    public List<Task> selectTaskList(Task task) {
        return taskMapper.selectTaskList(task);
    }

    /**
     * 修改任务
     *
     * @param task 任务
     * @return 结果
     */
    public int updateTask(Task task) {
        task.setUpdateTime(DateUtils.getNowDate());
        return taskMapper.updateTask(task);
    }

    /**
     * 批量删除任务
     *
     * @param taskIds 需要删除的任务主键
     * @return 结果
     */
    public int deleteTaskByTaskIds(Long[] taskIds) {
        return taskMapper.deleteTaskByTaskIds(taskIds);
    }

    /**
     * 删除任务信息
     *
     * @param taskId 任务主键
     * @return 结果
     */
    public int deleteTaskByTaskId(Long taskId) {
        return taskMapper.deleteTaskByTaskId(taskId);
    }

    // -------------------------------- tom add  --------------------------------
    private static final Logger l = LoggerFactory.getLogger(TaskService.class);
    @Autowired
    private FaultService faultService;
    @Autowired
    private TaskFaultService taskFaultService;

    /**
     * 新增任务
     *
     * @param tv 任务
     * @return 结果
     * todo:事务有问题。
     */
    @Transactional
    public AjaxResult insertTaskByTeacher(TaskVo tv) {
        l.info("task new task " + tv);
        // check
        if (tv == null) {
            return AjaxResult.error("TaskVo empty!");
        }
        if (!faultService.checkSimTypeOk(tv.getSimType())) {
            return AjaxResult.error("simType error!");
        }
        if (!Task.TASK_TYPE_TEACHER_ADD.equals(tv.getTaskType())) {
            return AjaxResult.error("taskType value error!");
        }
        if (StringUtils.isEmpty(tv.getName())) {
            return AjaxResult.error("name isEmpty!");
        }
        //
        tv.setCreateByUserId(SecurityUtils.getUserId());
        tv.setCreateBy(SecurityUtils.getUsername());
        tv.setCreateTime(DateUtils.getNowDate());
        tv.setUpdateBy(SecurityUtils.getUsername());
        tv.setUpdateTime(DateUtils.getNowDate());
        tv.setRemark("");
        List<FaultTreeVo> selectedData = tv.getSelectedData();
        taskMapper.insertTask(tv);
        if (selectedData != null) {
            List<FaultTreeVo> list = FaultService.flatten(selectedData);
            for (FaultTreeVo ftv : list) {
                if (ftv == null) {
                    continue;
                }
                if (!Fault.FAULT_TYPE_3.equals(ftv.getFaultType())) {
                    continue;
                }
                TaskFault tf = ftv.getTaskFault();
                // check
                if (tf == null) {
                    return AjaxResult.error("TaskFault empty!");
                }
                String flag = tf.getFlag();
                if (StringUtils.isEmpty(flag)) {
                    return AjaxResult.error("flag empty!");
                }
                if (TaskFault.UNKNOWN.equals(flag)) {
                    return AjaxResult.error("flag UNKNOWN!");
                }
                if (!TaskFault.YES.equals(flag) && !TaskFault.NO.equals(flag)) {
                    return AjaxResult.error("flag must YES or NO!");
                }
                // todo:选中数量限制

                // todo:故障部位冲突
                tf.setTaskId(tv.getTaskId());
                taskFaultService.insertOrUpdateTaskFault(tf);
            }
        }
        return AjaxResult.success();
    }

    /**
     * 为了新建，返回一个带tree结构的数据对象。
     *
     * @return 任务
     */
    public AjaxResult getInfoForAdd(String simType) {
        // check
        if (StringUtils.isEmpty(simType)) {
            return AjaxResult.error("simType empty!");
        }
        if (!faultService.checkSimTypeOk(simType)) {
            return AjaxResult.error("simType value error!");
        }
        // query obj.
        Fault q = new Fault();
        q.setSimType(simType);
        List<FaultTreeVo> ftList = (List<FaultTreeVo>) faultService.selectAllTreeViaSimType(simType).get(AjaxResult.DATA_TAG);
        TaskVo t = new TaskVo();
        t.setTaskId(Task.EMPTY_TASK_ID);
        t.setSelectedData(ftList);
        return AjaxResult.success(t);
    }

    /**
     * 新增任务附带选择故障
     *
     * @param task 任务
     * @return 结果
     */
    @Transactional
    public int insertTaskWithFault(Task task) {
        Long taskId = task.getTaskId();
        if (selectTaskByTaskId(taskId) != null) {
            // 已经存在
        }
        task.setCreateTime(DateUtils.getNowDate());
        task.setUpdateTime(DateUtils.getNowDate());
        return taskMapper.insertTask(task);
    }
}
